import { world, system, ItemStack } from "@minecraft/server";
import { handleItemDurability } from "./durability";
import { blockTags } from "./blockTags";
import { isValidPosition, getFillCommand } from "./fillCommand";
import { calculateFortuneMultiplier } from "./calculateFortuneMultiplier";

const blockDropTable = {
    "sns:pink_topaz_ore": "sns:raw_pink_topaz",
    "sns:garnet_ore": "sns:raw_garnet",
    "sns:malachite_ore": "sns:raw_malachite",
    "sns:alexandrite_ore": "sns:raw_alexandrite",
    "sns:amber_ore": "sns:raw_amber",
    "sns:axinite_ore": "sns:raw_axinite",
    "sns:tanzanite_ore": "sns:raw_tanzanite",
    "sns:kunzite_ore": "sns:raw_kunzite",
    "sns:ruby_ore": "sns:ruby",
    "sns:mithril_ore": "sns:mithril",
    "sns:jasper_ore": "sns:raw_jasper",
    "sns:deepslate_white_diamond_ore": "sns:white_diamond",
    "sns:adamantite_ore": "sns:raw_adamantite",
    "sns:corundum_ore": "sns:raw_corundum",
    "sns:quartz_ore": "sns:quartz",
    "sns:apatite_ore": "sns:apatite",
    "sns:fluorite_ore": "sns:raw_fluorite",
    "sns:pink_diamond_ore": "sns:pink_diamond",
    "sns:onyx_ore": "sns:onyx",
    "sns:infernal_ore": "sns:infernal",
    "sns:beryl_ore": "sns:raw_beryl",
    "sns:great_ruby_ore": "sns:great_ruby",
    "sns:dragon_scale_ore": "sns:dragon_scale",
    "sns:echo_ore": "sns:echo",
    "sns:deepslate_platinum_ore": "sns:raw_platinum",
    "sns:deepslate_sapphire_ore": "sns:sapphire",
    "sns:deepslate_skill_stone_ore": "sns:skill_stone",
    "sns:skill_stone_ore": "sns:skill_stone",
    "sns:cyanite_ore": "sns:raw_cyanite"


};

function handleBlockInteraction(event, blockTypeSet, functionName, itemTag) {
    const { block, source: player } = event;
    const { x, y, z } = block.location;
    const item = player.getComponent("inventory").container.getSlot(player.selectedSlotIndex).getItem();

    if (blockTypeSet.has(block.typeId) && item && item.hasTag(itemTag)) {
        handleItemDurability(player, item);
        event.cancel = true;
        player.runCommandAsync(`execute positioned ${x} ${y} ${z} run function ${functionName}`);
    }
}

world.beforeEvents.itemUseOn.subscribe(arg => {
    handleBlockInteraction(arg, blockTags.woodBlocksAxe, "axe", "sns:onuseon_axe");
    handleBlockInteraction(arg, blockTags.woodBlocksAxe, "stripped", "sns:is_tree_decapitator");
    handleBlockInteraction(arg, blockTags.woodBlocksAxe, "stripped_skill", "sns:onuseon_axe_skill");
    handleBlockInteraction(arg, blockTags.dirtBlocksShovel, "shovel", "sns:onuseon_shovel");
    handleBlockInteraction(arg, blockTags.dirtBlocksHoe, "hoe", "sns:onuseon_hoe");

});

world.beforeEvents.playerBreakBlock.subscribe(arg => {
    const { x, y, z } = arg.block.location;
    const { player } = arg;
    const item = player.getComponent("inventory").container.getSlot(player.selectedSlotIndex)?.getItem();

    if (item && item.hasTag("sns:is_tree_decapitator") && blockTags.woodandstrippedBlocks.has(arg.block.typeId)) {
        handleItemDurability(player, item);
        arg.cancel = true;
        player.runCommandAsync(`execute positioned ${x} ${y} ${z} run function filter`);
    }

    if (item && item.hasTag("sns:axe_skill") && blockTags.woodandstrippedBlocks.has(arg.block.typeId)) {
        handleItemDurability(player, item);
        arg.cancel = true;
        player.runCommandAsync(`execute positioned ${x} ${y} ${z} run function axe_skill`);
    }
});

world.beforeEvents.playerBreakBlock.subscribe(arg => {
    const { x, y, z } = arg.block.location;
    const { player } = arg;
    const position = player.location;
    const dimension = player.dimension;
    const item = player.getComponent("inventory").container.getSlot(player.selectedSlotIndex)?.getItem();

    if (item.hasTag("sns:is_hammer") && blockTags.pickaxeBlocks.has(arg.block.typeId)) {
        if (blockTags.ores.has(arg.block.typeId)) {
            const enchantableComp = item.getComponent("minecraft:enchantable");
            const fortuneLevel = enchantableComp?.getEnchantment("fortune")?.level || 0;
            const multiplier = calculateFortuneMultiplier(fortuneLevel);

            if (fortuneLevel > 0) {
                const dropItem = blockDropTable[arg.block.typeId];
                if (dropItem) {
                    system.run(() => {
                        for (let i = 0; i < multiplier; i++) {
                            dimension.spawnItem(new ItemStack(dropItem, 1), { x: x + 0.5, y: y + 0.5, z: z + 0.5 });
                        }
                    });
                }

            }

            return;
        }

        handleItemDurability(player, item);
        arg.cancel = true;

        for (let dx = -1; dx <= 1; dx++) {
            for (let dy = -1; dy <= 1; dy++) {
                for (let dz = -1; dz <= 1; dz++) {
                    const targetBlock = dimension.getBlock({ x: x + dx, y: y + dy, z: z + dz });

                    if (targetBlock && blockTags.pickaxeBlocks.has(targetBlock.typeId) && !blockTags.ores.has(targetBlock.typeId)) {
                        player.runCommandAsync(`setblock ${x + dx} ${y + dy} ${z + dz} air destroy`);
                    }
                }
            }
        }
    } else if (item) {
        if (item.hasTag("sns:pickaxe_skill") && blockTags.pickaxeBlocks.has(arg.block.typeId) && isValidPosition(dimension, position)) {
            const rotation = player.getRotation();
            const command = getFillCommand(rotation, x, y, z);
            if (command) {
                player.runCommandAsync(command);
            }
        }
    }
});

world.afterEvents.playerBreakBlock.subscribe(arg => {
    const { player } = arg;
    const item = player.getComponent("inventory").container.getSlot(player.selectedSlotIndex)?.getItem();
    if (item) {
        handleItemDurability(player, item);
    }
});